# SPDX-FileCopyrightText: © 2024 The "Whiteprints template context" contributors <whiteprints@pm.me>
#
# SPDX-License-Identifier: MIT

"""Top-level module."""

from typing import Final

from whiteprints_template_context.package_metadata import __version__


__all__: Final = ["__version__"]
"""Public module attributes."""
